/* @(#) $Id: cq.h,v 1.5 2002/10/11 02:20:15 leres Exp $ (LBL) */

struct cq_handle *cq_init(double, double);
void cq_destroy(struct cq_handle *);
int cq_enqueue(struct cq_handle *, double, void *);
void *cq_dequeue(struct cq_handle *, double);
int cq_size(struct cq_handle *);
int cq_max_size(struct cq_handle *);
#ifdef DEBUG
void cq_debug(struct cq_handle *, int);
void cq_dump(struct cq_handle *);
#endif
